==> jei-1.21.10-fabric-26.2.0.26-sources.jar: mezz/jei/library/plugins/debug/JeiDebugPlugin.java <==
package mezz.jei.library.plugins.debug;

import mezz.jei.api.IModPlugin;
import mezz.jei.api.JeiPlugin;
import mezz.jei.api.constants.ModIds;
import mezz.jei.api.constants.RecipeTypes;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.helpers.IJeiHelpers;
import mezz.jei.api.helpers.IPlatformFluidHelper;
import mezz.jei.api.ingredients.IIngredientTypeWithSubtypes;
import mezz.jei.api.registration.IAdvancedRegistration;
import mezz.jei.api.registration.IExtraIngredientRegistration;
import mezz.jei.api.registration.IGuiHandlerRegistration;
import mezz.jei.api.registration.IIngredientAliasRegistration;
import mezz.jei.api.registration.IModInfoRegistration;
import mezz.jei.api.registration.IModIngredientRegistration;
import mezz.jei.api.registration.IRecipeCatalystRegistration;
import mezz.jei.api.registration.IRecipeCategoryRegistration;
import mezz.jei.api.registration.IRecipeRegistration;
import mezz.jei.api.registration.ISubtypeRegistration;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.api.runtime.IJeiRuntime;
import mezz.jei.common.Internal;
import mezz.jei.common.config.DebugConfig;
import mezz.jei.common.gui.textures.Textures;
import mezz.jei.common.platform.IPlatformFluidHelperInternal;
import mezz.jei.common.platform.Services;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.RegistryUtil;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredient;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientHelper;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientListFactory;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientRenderer;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredient;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientHelper;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientListFactory;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientRenderer;
import net.minecraft.class_10591;
import net.minecraft.class_124;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_1887;
import net.minecraft.class_1893;
import net.minecraft.class_2246;
import net.minecraft.class_2378;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_472;
import net.minecraft.class_5321;
import net.minecraft.class_6880;
import net.minecraft.class_7924;
import net.minecraft.class_8059;
import net.minecraft.class_8060;
import net.minecraft.class_8786;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

@JeiPlugin
public class JeiDebugPlugin implements IModPlugin {
	private @Nullable DebugRecipeCategory<?> debugRecipeCategory;

	@Override
	public class_2960 getPluginUid() {
		return class_2960.method_60655(ModIds.JEI_ID, "debug");
	}

	@Override
	public void registerIngredients(IModIngredientRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			DebugIngredientHelper ingredientHelper = new DebugIngredientHelper();
			DebugIngredientRenderer ingredientRenderer = new DebugIngredientRenderer(ingredientHelper);
			registration.register(DebugIngredient.TYPE, Collections.emptyList(), ingredientHelper, ingredientRenderer, DebugIngredient.CODEC);

			if (DebugConfig.isCrashingTestIngredientsEnabled()) {
				ErrorIngredientHelper errorIngredientHelper = new ErrorIngredientHelper();
				ErrorIngredientRenderer errorIngredientRenderer = new ErrorIngredientRenderer(errorIngredientHelper);
				Collection<ErrorIngredient> errorIngredients = ErrorIngredientListFactory.create();
				registration.register(ErrorIngredient.TYPE, errorIngredients, errorIngredientHelper, errorIngredientRenderer, ErrorIngredient.CODEC);
			}
		}
	}

	@Override
	public void registerExtraIngredients(IExtraIngredientRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			registration.addExtraIngredients(DebugIngredient.TYPE, DebugIngredientListFactory.create(0, 10));
		}
	}

	@Override
	public void registerIngredientAliases(IIngredientAliasRegistration registration) {
		registration.addAlias(
			new class_1799(class_1802.field_8193),
			"jei.alias.panda.spawn.egg"
		);

		registration.addAlias(
			new class_1799(class_1802.field_8086),
			"jei.alias.villager.spawn.egg"
		);

		registration.addAliases(
			VanillaTypes.ITEM_STACK,
			List.of(
				new class_1799(class_1802.field_8615),
				new class_1799(class_1802.field_8077)
			),
			"nothing"
		);

		registration.addAliases(
			VanillaTypes.ITEM_STACK,
			List.of(
				new class_1799(class_1802.field_8303),
				new class_1799(class_1802.field_8603)
			),
			List.of("shiny", "valuable", "Expensive", "expansive", "extensive")
		);

		IPlatformFluidHelperInternal<?> fluidHelper = Services.PLATFORM.getFluidHelper();
		registerFluidAliases(registration, fluidHelper);
	}

	private <T> void registerFluidAliases(IIngredientAliasRegistration registration, IPlatformFluidHelper<T> fluidHelper) {
		@SuppressWarnings("deprecation")
		class_6880.class_6883<class_3611> water = class_3612.field_15910.method_40178();
		registration.addAliases(
			fluidHelper.getFluidIngredientType(),
			fluidHelper.create(water, fluidHelper.bucketVolume()),
			List.of("wet", "aqua", "sea", "ocean")
		);
	}

	@Override
	public void registerModInfo(IModInfoRegistration registration) {
		registration.addModAliases(ModIds.JEI_ID, "jei");
	}

	@Override
	public void registerCategories(IRecipeCategoryRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IGuiHelper guiHelper = jeiHelpers.getGuiHelper();
			IPlatformFluidHelper<?> platformFluidHelper = jeiHelpers.getPlatformFluidHelper();
			IIngredientManager ingredientManager = jeiHelpers.getIngredientManager();
			Textures textures = Internal.getTextures();
			this.debugRecipeCategory = new DebugRecipeCategory<>(guiHelper, platformFluidHelper, ingredientManager);
			registration.addRecipeCategories(
				debugRecipeCategory,
				new DebugFocusRecipeCategory<>(platformFluidHelper),
				new ObnoxiouslyLargeCategory(guiHelper, textures, ingredientManager),
				new ErrorRecipeCategory()
			);
		}
	}

	@Override
	public void registerRecipes(IRecipeRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			registration.addItemStackInfo(List.of(
				new class_1799(class_2246.field_10149),
				new class_1799(class_2246.field_10521),
				new class_1799(class_2246.field_10352),
				new class_1799(class_2246.field_10627),
				new class_1799(class_2246.field_10232),
				new class_1799(class_2246.field_10403)
				),
				class_2561.method_43471("description.jei.wooden.door.1"), // actually 2 lines
				class_2561.method_43471("description.jei.wooden.door.2"),
				class_2561.method_43471("description.jei.wooden.door.3")
			);

			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IPlatformFluidHelper<?> platformFluidHelper = jeiHelpers.getPlatformFluidHelper();
			registerFluidRecipes(registration, platformFluidHelper);
			registration.addIngredientInfo(new DebugIngredient(1), DebugIngredient.TYPE, class_2561.method_43470("debug"));
			registration.addIngredientInfo(new DebugIngredient(2), DebugIngredient.TYPE,
				class_2561.method_43470("debug colored").method_27692(class_124.field_1075),
				class_2561.method_43470("debug\\nSplit and colored").method_27692(class_124.field_1076),
				class_2561.method_43469("description.jei.debug.formatting.1", "various"),
				class_2561.method_43469("description.jei.debug.formatting.1", "various\\nsplit"),
				class_2561.method_43469("description.jei.debug.formatting.1", class_2561.method_43470("various colored").method_27692(class_124.field_1061)),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43470("various\\nsplit colored").method_27692(class_124.field_1062)
				),
				class_2561.method_43469("description.jei.debug.formatting.1", "\\nSplitting at the start"),
				class_2561.method_43469("description.jei.debug.formatting.1", "various all colored").method_27692(class_124.field_1061),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43469("description.jei.debug.formatting.3", "various").method_27692(class_124.field_1062)
				),
				class_2561.method_43469("description.jei.debug.formatting.2",
					class_2561.method_43470("multiple").method_27692(class_124.field_1065).method_27692(class_124.field_1056),
					class_2561.method_43470("various").method_27692(class_124.field_1061)
				).method_27692(class_124.field_1078),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43469("description.jei.debug.formatting.3",
						class_2561.method_43469("description.jei.debug.formatting.2",
							class_2561.method_43470("multiple").method_27692(class_124.field_1065).method_27692(class_124.field_1056),
							class_2561.method_43470("various").method_27692(class_124.field_1061)
						).method_27692(class_124.field_1062)
					)
				)
			);

			registration.addRecipes(DebugRecipeCategory.TYPE, List.of(
				new DebugRecipe(),
				new DebugRecipe()
			));

			registration.addRecipes(DebugFocusRecipeCategory.TYPE, List.of(
				new DebugRecipe()
			));

			class_2960 testRecipeWithoutTemplateId = class_2960.method_60655(ModIds.JEI_ID, "test_recipe_without_template");
			class_8786<class_8059> testRecipeWithoutTemplate = new class_8786<>(
				class_5321.method_29179(class_7924.field_52178, testRecipeWithoutTemplateId),
				new class_8060(
					Optional.empty(),
					class_1856.method_8101(class_1802.field_8279),
					Optional.of(class_1856.method_8101(class_1802.field_8512)),
					new class_10591(class_1802.field_8094)
				)
			);
			registration.addRecipes(RecipeTypes.SMITHING, List.of(
				testRecipeWithoutTemplate
			));

			registration.addRecipes(ObnoxiouslyLargeCategory.TYPE, List.of(new ObnoxiouslyLargeRecipe()));

			if (DebugConfig.isCrashingTestRecipesEnabled()) {
				registration.addRecipes(ErrorRecipeCategory.TYPE, Arrays.stream(ErrorRecipe.CrashType.values()).map(ErrorRecipe::new).toList());
			}
		}
	}

	private <T> void registerFluidRecipes(IRecipeRegistration registration, IPlatformFluidHelper<T> platformFluidHelper) {
		long bucketVolume = platformFluidHelper.bucketVolume();
		T fluidIngredient = platformFluidHelper.create(class_3612.field_15910.method_15785().method_40180(), bucketVolume);
		registration.addIngredientInfo(fluidIngredient, platformFluidHelper.getFluidIngredientType(), class_2561.method_43470("water"));

		fluidIngredient = platformFluidHelper.create(class_3612.field_15908.method_15785().method_40180(), 1);
		registration.addIngredientInfo(fluidIngredient, platformFluidHelper.getFluidIngredientType(), class_2561.method_43470("small amount of lava that should still show as 1 bucket"));
	}

	@Override
	public void registerGuiHandlers(IGuiHandlerRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IIngredientManager ingredientManager = jeiHelpers.getIngredientManager();

			registration.addGuiContainerHandler(class_472.class, new DebugBrewingStandScreenHandler());

			registration.addGhostIngredientHandler(class_472.class, new DebugGhostIngredientHandler<>(ingredientManager));
			registration.addGhostIngredientHandler(class_472.class, new DebugGhostIngredientHandlerTwo<>(ingredientManager));
		}
	}

	@Override
	public <T> void registerFluidSubtypes(ISubtypeRegistration registration, IPlatformFluidHelper<T> platformFluidHelper) {
		class_3611 water = class_3612.field_15910;
		IIngredientTypeWithSubtypes<class_3611, T> ingredientType = platformFluidHelper.getFluidIngredientType();
		FluidSubtypeHandlerTest<T> subtype = new FluidSubtypeHandlerTest<>(ingredientType);
		registration.registerSubtypeInterpreter(ingredientType, water, subtype);
	}

	@Override
	public void registerRecipeCatalysts(IRecipeCatalystRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IPlatformFluidHelper<?> fluidHelper = Services.PLATFORM.getFluidHelper();
			registerRecipeCatalysts(registration, fluidHelper);
		}
	}

	private <T> void registerRecipeCatalysts(IRecipeCatalystRegistration registration, IPlatformFluidHelper<T> fluidHelper) {
		long bucketVolume = fluidHelper.bucketVolume();

		registration.addCraftingStation(DebugRecipeCategory.TYPE, DebugIngredient.TYPE, new DebugIngredient(7));
		registration.addCraftingStation(DebugRecipeCategory.TYPE, fluidHelper.getFluidIngredientType(), fluidHelper.create(class_3612.field_15910.method_15785().method_40180(), bucketVolume));
		registration.addCraftingStation(DebugRecipeCategory.TYPE, class_1802.field_8600);

		RegistryUtil.getRegistry(class_7924.field_41197)
			.method_10220()
			.limit(300)
			.forEach(item -> {
				class_1799 catalystIngredient = new class_1799(item);
				if (!catalystIngredient.method_7960()) {
					registration.addCraftingStation(DebugRecipeCategory.TYPE, catalystIngredient);
				}
			});
	}

	@Override
	public void registerAdvanced(IAdvancedRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();

			jeiHelpers
				.getAllRecipeTypes()
				.filter(r -> r.getUid().method_12836().equals(ModIds.JEI_ID))
				.forEach(r -> registration.addRecipeCategoryDecorator(r, DebugCategoryDecorator.getInstance()));

			registration.addSimpleRecipeManagerPlugin(RecipeTypes.CRAFTING, new DebugSimpleRecipeManagerPlugin(jeiHelpers));
		}
	}

	@Override
	public void onRuntimeAvailable(IJeiRuntime jeiRuntime) {
		if (DebugConfig.isDebugModeEnabled()) {
			ErrorUtil.assertMainThread();
			class_2378<class_1887> registry = RegistryUtil.getRegistry(class_7924.field_41265);
			Optional<class_6880.class_6883<class_1887>> enchantment = registry.method_46746(class_1893.field_9124);
			assert enchantment.isPresent();
			if (debugRecipeCategory != null) {
				debugRecipeCategory.setRuntime(jeiRuntime);
			}
			IIngredientManager ingredientManager = jeiRuntime.getIngredientManager();
			ingredientManager.addIngredientsAtRuntime(DebugIngredient.TYPE, DebugIngredientListFactory.create(10, 20));
		}
	}
}

==> jei-1.21.11-fabric-27.3.0.9-sources.jar: mezz/jei/library/plugins/debug/JeiDebugPlugin.java <==
package mezz.jei.library.plugins.debug;

import mezz.jei.api.IModPlugin;
import mezz.jei.api.JeiPlugin;
import mezz.jei.api.constants.ModIds;
import mezz.jei.api.constants.RecipeTypes;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.gui.IRecipeLayoutDrawable;
import mezz.jei.api.gui.builder.ITooltipBuilder;
import mezz.jei.api.gui.buttons.IButtonState;
import mezz.jei.api.gui.buttons.IIconButtonController;
import mezz.jei.api.gui.inputs.IJeiUserInput;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.helpers.IJeiHelpers;
import mezz.jei.api.helpers.IPlatformFluidHelper;
import mezz.jei.api.ingredients.IIngredientTypeWithSubtypes;
import mezz.jei.api.recipe.advanced.IRecipeButtonControllerFactory;
import mezz.jei.api.registration.IAdvancedRegistration;
import mezz.jei.api.registration.IExtraIngredientRegistration;
import mezz.jei.api.registration.IGuiHandlerRegistration;
import mezz.jei.api.registration.IIngredientAliasRegistration;
import mezz.jei.api.registration.IModInfoRegistration;
import mezz.jei.api.registration.IModIngredientRegistration;
import mezz.jei.api.registration.IRecipeCatalystRegistration;
import mezz.jei.api.registration.IRecipeCategoryRegistration;
import mezz.jei.api.registration.IRecipeRegistration;
import mezz.jei.api.registration.ISubtypeRegistration;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.api.runtime.IJeiRuntime;
import mezz.jei.common.Internal;
import mezz.jei.common.config.DebugConfig;
import mezz.jei.common.gui.textures.Textures;
import mezz.jei.common.platform.IPlatformFluidHelperInternal;
import mezz.jei.common.platform.Services;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.RegistryUtil;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredient;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientHelper;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientListFactory;
import mezz.jei.library.plugins.debug.ingredients.DebugIngredientRenderer;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredient;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientHelper;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientListFactory;
import mezz.jei.library.plugins.debug.ingredients.ErrorIngredientRenderer;
import net.minecraft.class_10591;
import net.minecraft.class_124;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1856;
import net.minecraft.class_1887;
import net.minecraft.class_1893;
import net.minecraft.class_2246;
import net.minecraft.class_2378;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_472;
import net.minecraft.class_5321;
import net.minecraft.class_6880;
import net.minecraft.class_7924;
import net.minecraft.class_8059;
import net.minecraft.class_8060;
import net.minecraft.class_8786;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

@JeiPlugin
public class JeiDebugPlugin implements IModPlugin {
	private @Nullable DebugRecipeCategory<?> debugRecipeCategory;

	@Override
	public class_2960 getPluginUid() {
		return class_2960.method_60655(ModIds.JEI_ID, "debug");
	}

	@Override
	public void registerIngredients(IModIngredientRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			DebugIngredientHelper ingredientHelper = new DebugIngredientHelper();
			DebugIngredientRenderer ingredientRenderer = new DebugIngredientRenderer(ingredientHelper);
			registration.register(DebugIngredient.TYPE, Collections.emptyList(), ingredientHelper, ingredientRenderer, DebugIngredient.CODEC);

			if (DebugConfig.isCrashingTestIngredientsEnabled()) {
				ErrorIngredientHelper errorIngredientHelper = new ErrorIngredientHelper();
				ErrorIngredientRenderer errorIngredientRenderer = new ErrorIngredientRenderer(errorIngredientHelper);
				Collection<ErrorIngredient> errorIngredients = ErrorIngredientListFactory.create();
				registration.register(ErrorIngredient.TYPE, errorIngredients, errorIngredientHelper, errorIngredientRenderer, ErrorIngredient.CODEC);
			}
		}
	}

	@Override
	public void registerExtraIngredients(IExtraIngredientRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			registration.addExtraIngredients(DebugIngredient.TYPE, DebugIngredientListFactory.create(0, 10));
		}
	}

	@Override
	public void registerIngredientAliases(IIngredientAliasRegistration registration) {
		registration.addAlias(
			new class_1799(class_1802.field_8193),
			"jei.alias.panda.spawn.egg"
		);

		registration.addAlias(
			new class_1799(class_1802.field_8086),
			"jei.alias.villager.spawn.egg"
		);

		registration.addAliases(
			VanillaTypes.ITEM_STACK,
			List.of(
				new class_1799(class_1802.field_8615),
				new class_1799(class_1802.field_8077)
			),
			"nothing"
		);

		registration.addAliases(
			VanillaTypes.ITEM_STACK,
			List.of(
				new class_1799(class_1802.field_8303),
				new class_1799(class_1802.field_8603)
			),
			List.of("shiny", "valuable", "Expensive", "expansive", "extensive")
		);

		IPlatformFluidHelperInternal<?> fluidHelper = Services.PLATFORM.getFluidHelper();
		registerFluidAliases(registration, fluidHelper);
	}

	private <T> void registerFluidAliases(IIngredientAliasRegistration registration, IPlatformFluidHelper<T> fluidHelper) {
		@SuppressWarnings("deprecation")
		class_6880.class_6883<class_3611> water = class_3612.field_15910.method_40178();
		registration.addAliases(
			fluidHelper.getFluidIngredientType(),
			fluidHelper.create(water, fluidHelper.bucketVolume()),
			List.of("wet", "aqua", "sea", "ocean")
		);
	}

	@Override
	public void registerModInfo(IModInfoRegistration registration) {
		registration.addModAliases(ModIds.JEI_ID, "jei");
	}

	@Override
	public void registerCategories(IRecipeCategoryRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IGuiHelper guiHelper = jeiHelpers.getGuiHelper();
			IPlatformFluidHelper<?> platformFluidHelper = jeiHelpers.getPlatformFluidHelper();
			IIngredientManager ingredientManager = jeiHelpers.getIngredientManager();
			Textures textures = Internal.getTextures();
			this.debugRecipeCategory = new DebugRecipeCategory<>(guiHelper, platformFluidHelper, ingredientManager);
			registration.addRecipeCategories(
				debugRecipeCategory,
				new DebugFocusRecipeCategory<>(platformFluidHelper),
				new ObnoxiouslyLargeCategory(guiHelper, textures, ingredientManager),
				new ErrorRecipeCategory()
			);
		}
	}

	@Override
	public void registerRecipes(IRecipeRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			registration.addItemStackInfo(List.of(
				new class_1799(class_2246.field_10149),
				new class_1799(class_2246.field_10521),
				new class_1799(class_2246.field_10352),
				new class_1799(class_2246.field_10627),
				new class_1799(class_2246.field_10232),
				new class_1799(class_2246.field_10403)
				),
				class_2561.method_43471("description.jei.wooden.door.1"), // actually 2 lines
				class_2561.method_43471("description.jei.wooden.door.2"),
				class_2561.method_43471("description.jei.wooden.door.3")
			);

			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IPlatformFluidHelper<?> platformFluidHelper = jeiHelpers.getPlatformFluidHelper();
			registerFluidRecipes(registration, platformFluidHelper);
			registration.addIngredientInfo(new DebugIngredient(1), DebugIngredient.TYPE, class_2561.method_43470("debug"));
			registration.addIngredientInfo(new DebugIngredient(2), DebugIngredient.TYPE,
				class_2561.method_43470("debug colored").method_27692(class_124.field_1075),
				class_2561.method_43470("debug\\nSplit and colored").method_27692(class_124.field_1076),
				class_2561.method_43469("description.jei.debug.formatting.1", "various"),
				class_2561.method_43469("description.jei.debug.formatting.1", "various\\nsplit"),
				class_2561.method_43469("description.jei.debug.formatting.1", class_2561.method_43470("various colored").method_27692(class_124.field_1061)),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43470("various\\nsplit colored").method_27692(class_124.field_1062)
				),
				class_2561.method_43469("description.jei.debug.formatting.1", "\\nSplitting at the start"),
				class_2561.method_43469("description.jei.debug.formatting.1", "various all colored").method_27692(class_124.field_1061),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43469("description.jei.debug.formatting.3", "various").method_27692(class_124.field_1062)
				),
				class_2561.method_43469("description.jei.debug.formatting.2",
					class_2561.method_43470("multiple").method_27692(class_124.field_1065).method_27692(class_124.field_1056),
					class_2561.method_43470("various").method_27692(class_124.field_1061)
				).method_27692(class_124.field_1078),
				class_2561.method_43469("description.jei.debug.formatting.1",
					class_2561.method_43469("description.jei.debug.formatting.3",
						class_2561.method_43469("description.jei.debug.formatting.2",
							class_2561.method_43470("multiple").method_27692(class_124.field_1065).method_27692(class_124.field_1056),
							class_2561.method_43470("various").method_27692(class_124.field_1061)
						).method_27692(class_124.field_1062)
					)
				)
			);

			registration.addRecipes(DebugRecipeCategory.TYPE, List.of(
				new DebugRecipe(),
				new DebugRecipe()
			));

			registration.addRecipes(DebugFocusRecipeCategory.TYPE, List.of(
				new DebugRecipe()
			));

			class_2960 testRecipeWithoutTemplateId = class_2960.method_60655(ModIds.JEI_ID, "test_recipe_without_template");
			class_8786<class_8059> testRecipeWithoutTemplate = new class_8786<>(
				class_5321.method_29179(class_7924.field_52178, testRecipeWithoutTemplateId),
				new class_8060(
					Optional.empty(),
					class_1856.method_8101(class_1802.field_8279),
					Optional.of(class_1856.method_8101(class_1802.field_8512)),
					new class_10591(class_1802.field_8094)
				)
			);
			registration.addRecipes(RecipeTypes.SMITHING, List.of(
				testRecipeWithoutTemplate
			));

			registration.addRecipes(ObnoxiouslyLargeCategory.TYPE, List.of(new ObnoxiouslyLargeRecipe()));

			if (DebugConfig.isCrashingTestRecipesEnabled()) {
				registration.addRecipes(ErrorRecipeCategory.TYPE, Arrays.stream(ErrorRecipe.CrashType.values()).map(ErrorRecipe::new).toList());
			}
		}
	}

	private <T> void registerFluidRecipes(IRecipeRegistration registration, IPlatformFluidHelper<T> platformFluidHelper) {
		long bucketVolume = platformFluidHelper.bucketVolume();
		T fluidIngredient = platformFluidHelper.create(class_3612.field_15910.method_15785().method_40180(), bucketVolume);
		registration.addIngredientInfo(fluidIngredient, platformFluidHelper.getFluidIngredientType(), class_2561.method_43470("water"));

		fluidIngredient = platformFluidHelper.create(class_3612.field_15908.method_15785().method_40180(), 1);
		registration.addIngredientInfo(fluidIngredient, platformFluidHelper.getFluidIngredientType(), class_2561.method_43470("small amount of lava that should still show as 1 bucket"));
	}

	@Override
	public void registerGuiHandlers(IGuiHandlerRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();
			IIngredientManager ingredientManager = jeiHelpers.getIngredientManager();

			registration.addGuiContainerHandler(class_472.class, new DebugBrewingStandScreenHandler());

			registration.addGhostIngredientHandler(class_472.class, new DebugGhostIngredientHandler<>(ingredientManager));
			registration.addGhostIngredientHandler(class_472.class, new DebugGhostIngredientHandlerTwo<>(ingredientManager));
		}
	}

	@Override
	public <T> void registerFluidSubtypes(ISubtypeRegistration registration, IPlatformFluidHelper<T> platformFluidHelper) {
		class_3611 water = class_3612.field_15910;
		IIngredientTypeWithSubtypes<class_3611, T> ingredientType = platformFluidHelper.getFluidIngredientType();
		FluidSubtypeHandlerTest<T> subtype = new FluidSubtypeHandlerTest<>(ingredientType);
		registration.registerSubtypeInterpreter(ingredientType, water, subtype);
	}

	@Override
	public void registerRecipeCatalysts(IRecipeCatalystRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IPlatformFluidHelper<?> fluidHelper = Services.PLATFORM.getFluidHelper();
			registerRecipeCatalysts(registration, fluidHelper);
		}
	}

	private <T> void registerRecipeCatalysts(IRecipeCatalystRegistration registration, IPlatformFluidHelper<T> fluidHelper) {
		long bucketVolume = fluidHelper.bucketVolume();

		registration.addCraftingStation(DebugRecipeCategory.TYPE, DebugIngredient.TYPE, new DebugIngredient(7));
		registration.addCraftingStation(DebugRecipeCategory.TYPE, fluidHelper.getFluidIngredientType(), fluidHelper.create(class_3612.field_15910.method_15785().method_40180(), bucketVolume));
		registration.addCraftingStation(DebugRecipeCategory.TYPE, class_1802.field_8600);

		RegistryUtil.getRegistry(class_7924.field_41197)
			.method_10220()
			.limit(300)
			.forEach(item -> {
				class_1799 catalystIngredient = new class_1799(item);
				if (!catalystIngredient.method_7960()) {
					registration.addCraftingStation(DebugRecipeCategory.TYPE, catalystIngredient);
				}
			});
	}

	@Override
	public void registerAdvanced(IAdvancedRegistration registration) {
		if (DebugConfig.isDebugModeEnabled()) {
			IJeiHelpers jeiHelpers = registration.getJeiHelpers();

			jeiHelpers
				.getAllRecipeTypes()
				.filter(r -> r.getUid().method_12836().equals(ModIds.JEI_ID))
				.forEach(r -> registration.addRecipeCategoryDecorator(r, DebugCategoryDecorator.getInstance()));

			registration.addSimpleRecipeManagerPlugin(RecipeTypes.CRAFTING, new DebugSimpleRecipeManagerPlugin(jeiHelpers));

			registration.addRecipeButtonFactory(new IRecipeButtonControllerFactory() {
				@Override
				public <T> IIconButtonController createButtonController(IRecipeLayoutDrawable<T> recipeLayoutDrawable) {
					return new IIconButtonController() {
						@Override
						public void initState(IButtonState state) {
							state.setIcon(Internal.getTextures().getShapelessIcon());
						}

						@Override
						public boolean onPress(IJeiUserInput input) {
							return false;
						}

						@Override
						public void getTooltips(ITooltipBuilder tooltip) {
							tooltip.add(class_2561.method_43470("Debug Button"));
						}
					};
				}
			});
		}
	}

	@Override
	public void onRuntimeAvailable(IJeiRuntime jeiRuntime) {
		if (DebugConfig.isDebugModeEnabled()) {
			ErrorUtil.assertMainThread();
			class_2378<class_1887> registry = RegistryUtil.getRegistry(class_7924.field_41265);
			Optional<class_6880.class_6883<class_1887>> enchantment = registry.method_46746(class_1893.field_9124);
			assert enchantment.isPresent();
			if (debugRecipeCategory != null) {
				debugRecipeCategory.setRuntime(jeiRuntime);
			}
			IIngredientManager ingredientManager = jeiRuntime.getIngredientManager();
			ingredientManager.addIngredientsAtRuntime(DebugIngredient.TYPE, DebugIngredientListFactory.create(10, 20));
		}
	}
}
